"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var matchPattern = require('./build_arguments');

var CucumberExpression = function () {
  /**
   * @param expression
   * @param types Array of type name (String) or types (function). Functions can be a regular function or a constructor
   * @param parameterRegistry
   */
  function CucumberExpression(expression, types, parameterRegistry) {
    _classCallCheck(this, CucumberExpression);

    var parameterPattern = /\{([^}:]+)(:([^}]+))?}/g;
    var optionalPattern = /\(([^)]+)\)/g;

    this._expression = expression;
    this._parameters = [];
    var regexp = "^";
    var typeIndex = 0;
    var match = void 0;
    var matchOffset = 0;

    // Does not include (){} because they have special meaning
    expression = expression.replace(/([\\\^\[$.|?*+])/g, "\\$1");

    // Create non-capturing, optional capture groups from parenthesis
    expression = expression.replace(optionalPattern, '(?:$1)?');

    while ((match = parameterPattern.exec(expression)) !== null) {
      var parameterName = match[1];
      var typeName = match[3];
      // eslint-disable-next-line no-console
      if (typeName && typeof console !== 'undefined' && typeof console.error == 'function') {
        // eslint-disable-next-line no-console
        console.error("Cucumber expression parameter syntax {" + parameterName + ":" + typeName + "} is deprecated. Please use {" + typeName + "} instead.");
      }

      var type = types.length <= typeIndex ? null : types[typeIndex++];

      var parameter = void 0;
      if (type) {
        parameter = parameterRegistry.lookupByType(type);
      }
      if (!parameter && typeName) {
        parameter = parameterRegistry.lookupByTypeName(typeName);
      }
      if (!parameter) {
        parameter = parameterRegistry.lookupByTypeName(parameterName);
      }
      if (!parameter) {
        parameter = parameterRegistry.createAnonymousLookup(function (s) {
          return s;
        });
      }
      this._parameters.push(parameter);

      var text = expression.slice(matchOffset, match.index);
      var captureRegexp = getCaptureRegexp(parameter.captureGroupRegexps);
      matchOffset = parameterPattern.lastIndex;
      regexp += text;
      regexp += captureRegexp;
    }
    regexp += expression.slice(matchOffset);
    regexp += "$";
    this._regexp = new RegExp(regexp);
  }

  _createClass(CucumberExpression, [{
    key: "match",
    value: function match(text) {
      return matchPattern(this._regexp, text, this._parameters);
    }
  }, {
    key: "source",
    get: function get() {
      return this._expression;
    }
  }]);

  return CucumberExpression;
}();

function getCaptureRegexp(captureGroupRegexps) {
  if (captureGroupRegexps.length === 1) {
    return "(" + captureGroupRegexps[0] + ")";
  }

  var captureGroups = captureGroupRegexps.map(function (group) {
    return "(?:" + group + ")";
  });

  return "(" + captureGroups.join('|') + ")";
}

module.exports = CucumberExpression;