'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Parameter = require('./parameter');

var ParameterRegistry = function () {
  function ParameterRegistry() {
    _classCallCheck(this, ParameterRegistry);

    this._parametersByTypeName = new Map();
    this._parametersByCaptureGroupRegexp = new Map();
    this._parametersByConstructorName = new Map();

    var INTEGER_REGEXPS = [/-?\d+/, /\d+/];
    var FLOAT_REGEXPS = [/-?\d*\.?\d+/];

    this.addParameter(new Parameter('int', Number, INTEGER_REGEXPS, parseInt));
    this.addParameter(new Parameter('float', Number, FLOAT_REGEXPS, parseFloat));
  }

  _createClass(ParameterRegistry, [{
    key: 'lookupByType',
    value: function lookupByType(type) {
      if (typeof type === 'function') {
        return this.lookupByFunction(type);
      } else if (typeof type === 'string') {
        return this.lookupByTypeName(type);
      } else {
        throw new Error('Type must be string or function, but was ' + type + ' of type ' + (typeof type === 'undefined' ? 'undefined' : _typeof(type)));
      }
    }
  }, {
    key: 'lookupByFunction',
    value: function lookupByFunction(fn) {
      if (fn.name) {
        var prefix = fn.name[0];
        var looksLikeConstructor = prefix.toUpperCase() === prefix;

        var parameter = void 0;
        if (looksLikeConstructor) {
          parameter = this._parametersByConstructorName.get(fn.name);
        }
        if (!parameter) {
          var factory = function factory(s) {
            if (looksLikeConstructor) {
              return new fn(s);
            } else {
              return fn(s);
            }
          };
          return this.createAnonymousLookup(factory);
        } else {
          return parameter;
        }
      } else {
        return this.createAnonymousLookup(fn);
      }
    }
  }, {
    key: 'lookupByTypeName',
    value: function lookupByTypeName(typeName) {
      return this._parametersByTypeName.get(typeName);
    }
  }, {
    key: 'lookupByCaptureGroupRegexp',
    value: function lookupByCaptureGroupRegexp(captureGroupRegexp) {
      return this._parametersByCaptureGroupRegexp.get(captureGroupRegexp);
    }
  }, {
    key: 'createAnonymousLookup',
    value: function createAnonymousLookup(fn) {
      return new Parameter(null, null, [".+"], fn);
    }
  }, {
    key: 'addParameter',
    value: function addParameter(parameter) {
      this._parametersByConstructorName.set(parameter.constructorFunction.name, parameter);

      this._parametersByTypeName.set(parameter.typeName, parameter);

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = parameter.captureGroupRegexps[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var captureGroupRegexp = _step.value;

          this._parametersByCaptureGroupRegexp.set(captureGroupRegexp, parameter);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
  }, {
    key: 'parameters',
    get: function get() {
      return this._parametersByTypeName.values();
    }
  }]);

  return ParameterRegistry;
}();

module.exports = ParameterRegistry;